@@ -3,6 +3,7 @@ import copy
 from pathlib import Path
 import pandas as pd
 import time
+import random
 from typing import Sequence, Union, Tuple
 import psutil
 import logging
@@ -87,7 +88,7 @@ class UnsupervisedTrainer:
         if seed >= 0:
             self._set_seed(seed)
 
-        self.model: BaseCellModel = model.to(model.device)
+        self.model: BaseCellModel = model
 
         self.train_adata = self.test_adata = self.adata = adata
         if test_ratio > 0:
@@ -151,6 +152,7 @@ class UnsupervisedTrainer:
         torch.manual_seed(seed)
         torch.backends.cudnn.deterministic = True
         torch.backends.cudnn.benchmark = False
+        random.seed(seed)
         np.random.seed(seed)
         _logger.info(f'Set seed to {seed}.')
 
@@ -296,11 +298,12 @@ class UnsupervisedTrainer:
                     result = evaluate(adata = self.adata, **current_eval_kwargs)
                     if eval_result_log_path is not None:
                         with open(eval_result_log_path, 'a+') as f:
-                            # ckpt_dir, epoch, test_nll, best_ari, k_bet, ebm, time, seed
+                            # ckpt_dir, epoch, test_nll, best_ari, best_nmi, k_bet, ebm, time, seed
                             f.write(f'{Path(self.ckpt_dir).name}\t'
                                     f'{self.epoch}\t'
                                     f'{test_nll}\t'
                                     f'{result["best_ari"]}\t'
+                                    f'{result["best_nmi"]}\t'
                                     f'{result["k_bet"]}\t'
                                     f'{result["ebm"]}\t'
                                     f'{time.strftime("%m_%d-%H_%M_%S")}\t'
